
/********************************************
fin.h

libmawk changes (C) 2009-2012, Tibor 'Igor2' Palinkas;
based on mawk code coming with the below copyright:

copyright 1991, Michael D. Brennan

This is a source file for mawk, an implementation of
the AWK programming language.

Mawk is distributed without warranty under the terms of
the GNU General Public License, version 2, 1991.
********************************************/

/* buffered, splitting input (FIN, aka mawk_input_t) */

#ifndef  FIN_H
#define  FIN_H
/* structure to control input files */

enum {
	MAWK_INPF_MAIN               = 1,   /* part of main input stream if on */
	MAWK_INPF_EOF                = 2,   /* reached EOF */
	MAWK_INPF_START              = 4,   /* used when RS == "" */
	MAWK_INPF_NO_MORE            = 8,   /* there's no more input */
	MAWK_INPF_DEAD_NO_FREE       = 16,  /* static dead buffer - do not free */
	MAWK_INPF_NO_MORE_INPUTS     = 32   /* we are at the end of the input file list, impossible to have any more main input */
};

struct mawk_fin_s {
	mawk_vio_t *vf;
	char *buf;
	char *next;            /* start of the next record, within buf; anything before this is already returned in a previous call */
	int used;              /* how much bytes are in use in buf - this includes records already used up in the beginning of the buffer */
	int alloced;           /* total allocated size of the buf */
	int flags;
};


#define mawk_FIN_nomore (-2)

mawk_input_t *mawk_FINopen(mawk_state_t *MAWK, const char *name, int main_flag);
void mawk_FINclose(mawk_state_t *, mawk_input_t *);
void mawk_FINsemi_close(mawk_state_t *, mawk_input_t *);
unsigned mawk_fillbuff(mawk_state_t *, mawk_input_t *, char *, unsigned, int interactive);

/* open an existing vio vf as FIN */
mawk_input_t *mawk_FIN_vio_open(mawk_state_t *MAWK, mawk_vio_t *vf, int main_flag);

/* open "stdin" as mawk_input_t */
mawk_input_t *mawk_fin_fd0_open(mawk_state_t *MAWK, int main_flag);

/* execution: */
void mawk_FINopen_main(mawk_state_t * MAWK);
char *mawk_FINgets(mawk_state_t *, mawk_input_t *, unsigned *);


#endif /* FIN_H */
